<form class="referral" action="{% url anafero_create_referral %}" method="post">
    {% csrf_token %}
    <input type="hidden" name="redirect_to" id="redirect_to" value="{{ url }}" />
    <input type="hidden" name="obj_ct_pk" id="obj_ct_pk" value="{{ obj_ct.pk }}" />
    <input type="hidden" name="obj_pk" id="obj_pk" value="{{ obj.pk }}" />
    
    <button type="submit">Create Referral Link</button>
</form>